#ifdef HAVE_S390_MIN_Z196_ZARCH_ASM_SUPPORT
# define USE_TRUNC_BUILTIN 1
# define USE_TRUNCF_BUILTIN 1
# define USE_TRUNCL_BUILTIN 1
# if __GNUC_PREREQ (8, 0)
#  define USE_TRUNCF128_BUILTIN 1
# else
#  define USE_TRUNCF128_BUILTIN 0
# endif
#else
# define USE_TRUNC_BUILTIN 0
# define USE_TRUNCF_BUILTIN 0
# define USE_TRUNCL_BUILTIN 0
# define USE_TRUNCF128_BUILTIN 0
#endif
