#ifdef _ARCH_PWR5X
# define USE_TRUNCF_BUILTIN 1
# define USE_TRUNC_BUILTIN 1
#else
# define USE_TRUNCF_BUILTIN 0
# define USE_TRUNC_BUILTIN 0
#endif
#define USE_TRUNCL_BUILTIN 0
#define USE_TRUNCF128_BUILTIN 0
