#if __GNUC_PREREQ (13, 0)
# define USE_FMIN_BUILTIN 1
# define USE_FMINF_BUILTIN 1
#else
# define USE_FMIN_BUILTIN 0
# define USE_FMINF_BUILTIN 0
#endif

#define USE_FMINL_BUILTIN 0
#define USE_FMINF128_BUILTIN 0
